package cn.iocoder.yudao.module.member.mq.consumer.score.core;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreLog.MemberUserScoreLogDO;
import cn.iocoder.yudao.module.member.dal.dataobject.scoreRule.ScoreRuleDO;
import cn.iocoder.yudao.module.member.dal.redis.scoreRule.ScoreRuleRedisDao;
import cn.iocoder.yudao.module.member.enums.ScoreRuleStatusEnum;
import cn.iocoder.yudao.module.member.enums.ScoreRuleTypeEnum;
import cn.iocoder.yudao.module.member.enums.TransportTypeEnum;
import cn.iocoder.yudao.module.member.service.scoreLog.MemberUserScoreLogService;
import cn.iocoder.yudao.module.member.service.scoreRule.ScoreRuleService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author zhangfeng
 */
public abstract class AbstractScoreRuleStrategy implements ScoreRuleStrategy {

    protected ScoreRuleService scoreRuleService;
    protected MemberUserScoreLogService memberUserScoreLogService;

    @Autowired
    public AbstractScoreRuleStrategy(ScoreRuleService scoreRuleService, MemberUserScoreLogService memberUserScoreLogService) {
        this.scoreRuleService = scoreRuleService;
        this.memberUserScoreLogService = memberUserScoreLogService;
    }

    @Resource
    ScoreRuleRedisDao scoreRuleRedisDao;

    protected ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType) {
        ScoreRuleDO enableScoreRule;
        enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType);
        if (enableScoreRule == null) {
            enableScoreRule = scoreRuleService.selectOne(new LambdaQuery<ScoreRuleDO>()
                    .eq(ScoreRuleDO::getType, scoreRuleType.getValue())
                    .eq(ScoreRuleDO::getStatus, ScoreRuleStatusEnum.ENABLED.getValue()));
            if (enableScoreRule == null) {
                return null;
            }
            // 校验一下活动开始结束时间
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
            scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
        } else {
            if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                return null;
            }
        }
        return enableScoreRule;
    }

    protected ScoreRuleDO getEnableScoreRuleByType(ScoreRuleTypeEnum scoreRuleType, TransportTypeEnum transportType) {
        ScoreRuleDO enableScoreRule;
        switch (scoreRuleType) {
            case ORDER_V:
                enableScoreRule = scoreRuleRedisDao.getEnableScoreRule(scoreRuleType, transportType);
                if (enableScoreRule == null) {
                    enableScoreRule = scoreRuleService.getEnabledOrderVScoreRuleByTransportType(transportType.getValue());
                    if (enableScoreRule == null) {
                        return null;
                    }
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                    scoreRuleRedisDao.setEnableScoreRule(enableScoreRule);
                } else {
                    if (!enableScoreRule.getEndTime().after(new Date()) || !enableScoreRule.getStartTime().before(new Date())) {
                        return null;
                    }
                }
                return enableScoreRule;
            case RECOMMEND:
            case REGISTER:
                return getEnableScoreRuleByType(scoreRuleType);
        }
        return null;
    }

    /**
     * 计算用户在某一活动获得积分总数
     * @param scoreRuleId
     * @param userId
     * @return
     */
    protected Integer getUserScoreTotalCount(Long scoreRuleId, Long userId) {
        Integer userScoreTotalCount = 0;
        LambdaQueryWrapper<MemberUserScoreLogDO> scoreLogQueryWrapper = new LambdaQueryWrapper<>();
        scoreLogQueryWrapper.eq(MemberUserScoreLogDO::getMemberId, userId)
                .eq(MemberUserScoreLogDO::getRuleId, scoreRuleId);
        List<MemberUserScoreLogDO> memberUserScoreLogs = memberUserScoreLogService.selectList(scoreLogQueryWrapper);
        if (CollUtil.isNotEmpty(memberUserScoreLogs)) {
            for (MemberUserScoreLogDO memberUserScoreLog : memberUserScoreLogs) {
                userScoreTotalCount += memberUserScoreLog.getScoreCount();
            }
        }
        return userScoreTotalCount;
    }
}

